#include "stdafx.h"
#include "AgentDLL.h"
#include "tchar.h"
#include <shlobj.h>
#include <string>
#include <atlchecked.h>
#include "XMLUtil.h"
#include "tinyxml.h"
#include "HttpClient/httpclient.h"
#include <io.h>
#include <atlpath.h>

const TCHAR *AGENTDLL_DEFAULT_PATH = _T("NHN Corporation\\NaverToolsUpdater\\UpgradeInterface");
const TCHAR *VERSION_XML = _T("Version.XML");
const TCHAR *VERSION_XML_PARENT_TAG = _T("agentdll");
const TCHAR *VERSION_XML_VERSION_TAG = _T("version");
const TCHAR *VERSION_XML_DLLNAME_TAG = _T("dllname");
const TCHAR *VERSION_XML_DLLURL_TAG = _T("url");
const TCHAR *AGENTDLL_ERROR_NODE = _T("error");
const TCHAR *AGENTDLL_ID_NODE = _T("id");
const TCHAR *AGENTDLL_MESSAGE_NODE = _T("message");


const int REQUEST_QUERY_ERROR_CODE = -1;

const TCHAR *REQUEST_QUERY_INSTALL_ERROR = _T("install.error");
const TCHAR *REQUEST_QUERY_UNINSTALL_ERROR = _T("uninstall.error");
const TCHAR *REQUEST_QUERY_AGREEMENT_SHOWN = _T("agreement.shown");
const TCHAR *REQUEST_QUERY_AGREEMENT_ISSHOW = _T("agreement.isshow");
const TCHAR *REQUEST_QUERY_AGREEMENT_URL = _T("agreement.url");
const TCHAR *REQUEST_QUERY_GUID = _T("guid");
const TCHAR *REQUEST_QUERY_ISINSTALLED = _T("isinstalled");
const TCHAR *REQUEST_QUERY_UNINSTALLER_URL = _T("uninstaller.url");
const TCHAR *REQUEST_QUERY_VERSION_LOCAL = _T("version.local");
const TCHAR *REQUEST_QUERY_VERSION_RECENT = _T("version.recent");
const TCHAR *REQUEST_QUERY_VERSION_ISCHANGED = _T("version.ischanged");
const TCHAR *REQUEST_QUERY_UPGRADER_RUN = _T("upgrader.run");
const TCHAR *REQUEST_QUERY_UPGRADER_STATUS = _T("upgrader.status");

BOOL IsInstalled();
int GetServiceID (LPTSTR pGUID, int bufferLen);
BOOL RunUpgrader (DWORD runType);
BOOL GetUpdateStatus (int& status);
int GetUnistallerInfo (TCHAR* path, int pathLen, int& unistallerType);
int GetLocalVersion (TCHAR* version, int bufferLen);
int GetRecentVersion (TCHAR* version, int bufferLen);
BOOL IsVersionChanged ();

AgentDLLWrapper::AgentDLLWrapper(const TCHAR *path) : m_module(NULL), m_dllpath(path), m_open(FALSE)
{

}

AgentDLLWrapper::~AgentDLLWrapper()
{
	Close();
}

BOOL AgentDLLWrapper::Open( const TCHAR *filename)
{
	if( m_open )
		return TRUE;

	m_module = ::LoadLibrary(filename);
	if( NULL == m_module )
	{
		return FALSE;
	}

	m_open = TRUE;
	m_pfn_Request			= (Func_Request)GetProcAddress(m_module, "Request");
	return TRUE;
}

BOOL AgentDLLWrapper::Close()
{
	BOOL ret = FALSE;
	if( m_module && m_open )
	{
		ret = ::FreeLibrary(m_module);
	}
	
	return ret;
}

void MakeXML(CString &strXML, CString elementXML)
{
	strXML += _T("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\r\n");
	strXML += _T("<root>\r\n");
	strXML += elementXML;
	strXML += _T("</root>");
}

int MakeQueryXML(LPCTSTR qid, LPCTSTR valueXML, char* response, int response_length)
{
	CString strElement;
	strElement = _T("<query id=\"");
	strElement += qid;
	strElement += _T("\">");
	strElement += _T("\r\n");
	if( valueXML )
	{
		strElement += valueXML;
		strElement += _T("\r\n");
	}
	strElement += _T("</query>\r\n");
	CString strXML;
	MakeXML(strXML, strElement);
	USES_CONVERSION;
	strcpy_s(response, response_length, T2A(strXML.GetString()));
	return response_length;
}

int MakeResponseElement(LPCTSTR qid, LPCTSTR value, char* response, int response_length)
{
	CString strElement;
	XMLUtil::WriteElementValue(strElement, qid, value);
	CString strXML;
	MakeXML(strXML, strElement);
	USES_CONVERSION;
	strcpy_s(response, response_length, T2A(strXML.GetString()));
	return response_length;
}

int MakeResponseElement(LPCTSTR qid, LONG value, char* response, int response_length)
{
	CString strElement;
	XMLUtil::WriteElementValue(strElement, qid, value);
	CString strXML;
	MakeXML(strXML, strElement);
	USES_CONVERSION;
	strcpy_s(response, response_length, T2A(strXML.GetString()));
	return response_length;
}

int MakeResponseElement(LPCTSTR qid, BOOL value, char* response, int response_length)
{
	CString strElement;
	XMLUtil::WriteElementValue(strElement, qid, value);
	CString strXML;
	MakeXML(strXML, strElement);
	USES_CONVERSION;
	strcpy_s(response, response_length, T2A(strXML.GetString()));
	return response_length;
}

void MakeQueryElement(LPCTSTR qid, LONG value, CString &element)
{
	XMLUtil::WriteElementValue(element, qid, value);
}

void MakeQueryElement(LPCTSTR qid, BOOL value, CString &element)
{
	XMLUtil::WriteElementValue(element, qid, value);
}

BOOL IsErrorXML( TiXmlElement *xml_element, int &error_code, CString &error_msg )
{
	error_code = 0; //  ڵ ʱȭ
	TiXmlElement *pElement = xml_element->FirstChildElement( CT2CA(AGENTDLL_ERROR_NODE) );
	if(pElement)
	{
		LONG code;
		if( TRUE == XMLUtil::GetValue(pElement, "id", code) )
		{
			error_code = code;
		}
		TCHAR data[4096];
		if (TRUE == XMLUtil::GetValue(pElement, "message", data, 4096) )
		{
			error_msg = data;
		}

		return TRUE;
	}

	return FALSE;
}

BOOL AgentDLLWrapper::SendRequest( const char *query, char *response, int responseSize )
{
	if( REQUEST_QUERY_ERROR_CODE == m_pfn_Request( query, response, responseSize ) )
	{
		return FALSE;
	}

	return TRUE;
}

int GetParseAgreementUrlXML( const char *response, AgreementUrlList &urlList, int &error_code, CString &error_msg )
{
	TiXmlDocument doc;
	doc.Parse(response);
	TiXmlElement* pElement = doc.FirstChildElement("root");
	if(pElement == NULL)
		return -1;

	if( IsErrorXML(pElement, error_code, error_msg) == TRUE )
		return -1;

	pElement = pElement->FirstChildElement( CT2CA(REQUEST_QUERY_AGREEMENT_URL) );
	if(pElement == NULL)
		return -1;

	//int urlNum; [] BOOL int typedef Ǿ ־ ߸ GetValue ȣȴ.
	LONG urlNum;
	if( FALSE == XMLUtil::GetValue(pElement, "num", urlNum) )
		return -1;

	TCHAR url[1024*8];
	for( int i = 0; i < urlNum; i++ )
	{
		CString urlCount;
		urlCount.Format(_T("url.%d"), i);
		if( FALSE == XMLUtil::GetValue(pElement, CT2CA(urlCount), url, 1024*8  ) )
			return -1;

		AgreementUrlItem item;
		item.url = url;

		urlList.push_back(item);
	}

	return urlList.size();
}

int GetParseUninstallerInfoXML( const char *response, int &type, TCHAR *url, int urlsize, TCHAR *param, int paramLen, int &error_code, CString &error_msg )
{
	TiXmlDocument doc;
	doc.Parse(response);
	TiXmlElement* pElement = doc.FirstChildElement("root");
	if(pElement == NULL)
		return -1;

	if( IsErrorXML(pElement, error_code, error_msg) == TRUE )
		return -1;

	pElement = pElement->FirstChildElement( CT2CA(REQUEST_QUERY_UNINSTALLER_URL) );
	if(pElement == NULL)
		return -1;

	if( FALSE == XMLUtil::GetValue(pElement, "type", (LONG&)type) )
		return -1;

	if( FALSE == XMLUtil::GetValue(pElement, "url", url, urlsize ) )
		return -1;

	if( param )
	{
		if( FALSE == XMLUtil::GetValue(pElement, "parameter", param, paramLen))
		{
			//parameter   Ѿ 찡 κ̹Ƿ   ó
			param[0] = '\0';
		}
	}

	return 1;
}

int GetParseXML( const TCHAR *id, CString &data, const char *response, int &error_code, CString &error_msg )
{
	TiXmlDocument doc;
	
//	CString test_xml = _T("<?xml version=\"1.0\" encoding=\"UTF-8\" ?><root><error><id>1</id><message>test</message></root>");
//	CT2CA xmlAscii(test_xml);

//	doc.Parse(xmlAscii);
	doc.Parse(response);
	TiXmlElement* pElement = doc.FirstChildElement("root");
	if(pElement == NULL)
		return -1;

	if( TRUE == IsErrorXML( pElement, error_code, error_msg ) )
		return -1;

	CT2CA query_id(id);

	TiXmlNode *pNode = pElement->FirstChild(query_id);
	if(pNode == NULL)
		return -1;

	pNode = pNode->FirstChild();
	if(pNode == NULL)
		return -1;

	data = pNode->Value();
	return 1;
}

BOOL AgentDLLWrapper::DefaultRequest( const TCHAR *queryid, const TCHAR *value, CString &data, int *error_code, CString *error_msg ) 
{
	char query[1024*8];
	char response[1024*8];
	MakeQueryXML( queryid, value, query, 1024*8 );

	if( FALSE == SendRequest( query, response, 1024*8 ) )
	{
		if( error_code )
			*error_code = -1;
		return FALSE;
	}

	CString tmp_error_msg;
	int tmp_error_code = 0;
	int ret;

	ret = GetParseXML( queryid, data, response, tmp_error_code, tmp_error_msg );

	if( error_code )
		*error_code = tmp_error_code;
	if( error_msg )
		*error_msg = tmp_error_msg;

	if( -1 == ret || (error_code && 0 != *error_code ) )
		return FALSE;

	return TRUE;
};

BOOL AgentDLLWrapper::RequestIsInstalled(int *error_code, CString *error_msg)
{
	CString data;
	if( FALSE == DefaultRequest( REQUEST_QUERY_ISINSTALLED, NULL, data, error_code, error_msg ) )
		return FALSE;

	return (_tcsicmp(data, _T("true")) == 0);
}

BOOL AgentDLLWrapper::RequestGuid(TCHAR *guid, int guidSize,int *error_code, CString *error_msg)
{
	CString data;
	if( FALSE == DefaultRequest( REQUEST_QUERY_GUID, NULL, data, error_code, error_msg ) )
		return FALSE;

	Checked::tcscpy_s( guid, guidSize, data );
	return TRUE;
}

BOOL AgentDLLWrapper::RequestIsVersionChanged(int *error_code, CString *error_msg)
{
	CString data;
	if( FALSE == DefaultRequest( REQUEST_QUERY_VERSION_ISCHANGED, NULL, data, error_code, error_msg ) )
		return FALSE;

	return (_tcsicmp(data, _T("true")) == 0);
}

BOOL AgentDLLWrapper::RequestRunUpgrader(DWORD runType,int *error_code, CString *error_msg)
{
	CString data;
	CString value;
	MakeQueryElement( _T("value"), (LONG)runType, value );

	if( FALSE == DefaultRequest( REQUEST_QUERY_UPGRADER_RUN, value, data, error_code, error_msg ) )
		return FALSE;

	return (_tcsicmp(data, _T("true")) == 0);
}

BOOL AgentDLLWrapper::RequestGetUpdateStatus(int &status, int *error_code, CString *error_msg)
{
	CString data;
	if( FALSE == DefaultRequest( REQUEST_QUERY_UPGRADER_STATUS, NULL, data, error_code, error_msg ) )
		return FALSE;

	USES_CONVERSION;
	status = atoi(T2A(data));
	return TRUE;
}

BOOL AgentDLLWrapper::RequestAgreementShown(BOOL bShow, int *error_code, CString *error_msg)
{
	CString data;
	CString value;
	MakeQueryElement( _T("value"), (BOOL)bShow, value );

	if( FALSE == DefaultRequest( REQUEST_QUERY_AGREEMENT_SHOWN, value, data, error_code, error_msg ) )
		return FALSE;

	return (_tcsicmp(data, _T("true")) == 0);
}

BOOL AgentDLLWrapper::RequestIsAgreementShow(int *error_code, CString *error_msg)
{
	CString data;
	if( FALSE == DefaultRequest( REQUEST_QUERY_AGREEMENT_ISSHOW, NULL, data, error_code, error_msg ) )
		return FALSE;

	return (_tcsicmp(data, _T("true")) == 0);
}

int  AgentDLLWrapper::RequestGetUnistallerInfo(TCHAR* path, int pathLen, TCHAR *param, int paramLen, int &uninstallerType, int *error_code, CString *error_msg)
{
	char query[1024*8];
	char response[1024*8];
	MakeQueryXML( REQUEST_QUERY_UNINSTALLER_URL, NULL, query, 1024*8 );

	if( FALSE == SendRequest( query, response, 1024*8 ) )
		return FALSE;

	CString tmp_error_msg;
	int tmp_error_code;
	int ret;

	ret = GetParseUninstallerInfoXML( response, uninstallerType, path, pathLen, param, paramLen, tmp_error_code, tmp_error_msg );
	if( error_code )
		*error_code = tmp_error_code;
	if( error_msg )
		*error_msg = tmp_error_msg;

	if( -1 == ret || (error_code && 0 != *error_code ) )
		return -1;

	return ret;
}

int AgentDLLWrapper::RequestAgreementUrlList(AgreementUrlList &urlList, int *error_code, CString *error_msg)
{
	char query[1024*8];
	char response[1024*8];
	MakeQueryXML( REQUEST_QUERY_AGREEMENT_URL, NULL, query, 1024*8 );

	if( FALSE == SendRequest( query, response, 1024*8 ) )
		return FALSE;
	
	CString data;
	CString tmp_error_msg;
	int tmp_error_code = 0;

	int ret = GetParseAgreementUrlXML( response, urlList, tmp_error_code, tmp_error_msg ); 

	if( error_code )
		*error_code = tmp_error_code;
	if( error_msg )
		*error_msg = tmp_error_msg;

	if( -1 == ret || (error_code && 0 != *error_code ) )
		return -1;

	return ret;
}

BOOL AgentDLLWrapper::RequestInstallError(int *error_code, CString *error_msg)
{
	CString data;
	if( FALSE == DefaultRequest( REQUEST_QUERY_INSTALL_ERROR, NULL, data, error_code, error_msg ) )
		return TRUE;

	return (_tcsicmp(data, _T("true")) == 0);
}

BOOL AgentDLLWrapper::RequestUninstallError(int *error_code, CString *error_msg)
{
	CString data;
	if( FALSE == DefaultRequest( REQUEST_QUERY_UNINSTALL_ERROR, NULL, data, error_code, error_msg ) )
		return TRUE;

	return (_tcsicmp(data, _T("true")) == 0);
}

BOOL AgentDLLWrapper::GetInstallError(int *error_code, CString *error_msg)
{
	if( FALSE == Open(m_dllpath) )
		return TRUE;

	return RequestInstallError(error_code, error_msg);
}

BOOL AgentDLLWrapper::GetUninstallError(int *error_code, CString *error_msg)
{
	if( FALSE == Open(m_dllpath) )
		return TRUE;

	return RequestUninstallError(error_code, error_msg);
}

BOOL AgentDLLWrapper::GetGuid(TCHAR *guid, int guidSize, int *error_code, CString *error_msg)
{
	if( FALSE == Open(m_dllpath) )
		return FALSE;

	return RequestGuid(guid, guidSize, error_code, error_msg);
}

Func_Request AgentDLLWrapper::GetFuncRequest()
{
	return m_pfn_Request;
}

BOOL AgentDLLWrapper::GetInstalledStatus(NAVERAPP_INSTALL_STATUS &status, int *error_code, CString *error_msg)
{
	if( FALSE == Open(m_dllpath) )
		return FALSE;

	status = LatestVersion;
	if( FALSE == RequestIsInstalled(error_code, error_msg) )
		status = NotInstalled;
	else if( TRUE == RequestIsVersionChanged(error_code, error_msg) )
		status = OldVersion;

	if( error_code && 0 != *error_code )
		return FALSE;

	return TRUE;
}

int AgentDLLWrapper::GetAgreementUrlList(AgreementUrlList &urlList, int *error_code, CString *error_msg)
{
	if( FALSE == Open(m_dllpath) )
		return FALSE;

	return RequestAgreementUrlList(urlList, error_code, error_msg);
}

BOOL AgentDLLWrapper::AgreementShown(BOOL bShow, int *error_code, CString *error_msg)
{
	if( FALSE == Open(m_dllpath) )
		return FALSE;

	return RequestAgreementShown(bShow, error_code, error_msg);
}

BOOL AgentDLLWrapper::IsAgreementShow(int *error_code, CString *error_msg)
{
	if( FALSE == Open(m_dllpath) )
		return FALSE;

	return RequestIsAgreementShow(error_code, error_msg);
}

BOOL AgentDLLWrapper::RunUpgrader(DWORD runType, int *error_code, CString *error_msg)
{
	if( FALSE == Open(m_dllpath) )
		return FALSE;

	return RequestRunUpgrader(runType, error_code, error_msg);
}

BOOL AgentDLLWrapper::GetUpdateStatus(int &status, int *error_code, CString *error_msg)
{
	if( FALSE == Open(m_dllpath) )
		return FALSE;

	return RequestGetUpdateStatus(status, error_code, error_msg);
}

int AgentDLLWrapper::GetUninstallerInfo( TCHAR* path, int pathLen, TCHAR *param, int paramLen, int &uninstallerType, int *error_code, CString *error_msg)
{
	if( FALSE == Open(m_dllpath) )
		return -1;

	return RequestGetUnistallerInfo( path, pathLen, param, paramLen, uninstallerType, error_code, error_msg );
}

const CString GetUpgradeInterfaceFolderName()
{
	TCHAR szPath[1024];

	if(FAILED(SHGetFolderPath(NULL, 
		CSIDL_COMMON_APPDATA, 
		NULL, 
		0, 
		szPath))) 
	{
		return _T("");
	}

	ATLPath::Append( szPath, AGENTDLL_DEFAULT_PATH );

	TCHAR username[1024];
	DWORD username_size = GetEnvironmentVariable( _T("USERNAME"), username, 1024 );
	username[username_size] = 0;

	ATLPath::Append( szPath, username );
	CString upgradeInterfaceFolder = szPath;
	return upgradeInterfaceFolder;
}

const CString GetGuidInterfaceFolderName( const TCHAR *guid )
{
	CString guidInterfaceFolder = GetUpgradeInterfaceFolderName();
	guidInterfaceFolder += '\\';
	guidInterfaceFolder += guid;
	guidInterfaceFolder += '\\';
	return guidInterfaceFolder;
};

BOOL GetDLLVersionAndPath( CString path, CString &version, CString &dllPath )
{
	path += VERSION_XML;

	TiXmlDocument xmlDoc;// = new TiXmlDocument;

	CT2CA filename(path);
	if( false == xmlDoc.LoadFile( filename, TIXML_ENCODING_UTF8 ) )
		return FALSE;

	TiXmlElement* root = xmlDoc.RootElement();
	if( NULL == root )
		return FALSE;

	TiXmlElement* tag = (TiXmlElement*)root->FirstChild( CT2CA(VERSION_XML_PARENT_TAG) );
	if( NULL == tag )
		return FALSE;

	TiXmlElement* versionElement = (TiXmlElement*)tag->FirstChild( CT2CA(VERSION_XML_VERSION_TAG) );
	if( NULL == versionElement )
		return FALSE;

	version = versionElement->GetText();

	TiXmlElement* dllnameElement = (TiXmlElement*)tag->FirstChild( CT2CA(VERSION_XML_DLLNAME_TAG) );
	if( NULL == dllnameElement )
		return FALSE;

	dllPath = dllnameElement->GetText();
	//delete xmlDoc;

	return TRUE;
}

BOOL GetDLLVersionAndUrl( BYTE *data, CString &version, CString &url )
{
	TiXmlDocument xmlDoc;// = new TiXmlDocument;

	xmlDoc.Parse((const char *)data, NULL, TIXML_ENCODING_UTF8 );

	TiXmlElement* root = xmlDoc.RootElement();
	if( NULL == root )
		return FALSE;

	TiXmlElement* tag = (TiXmlElement*)root->FirstChild( CT2CA(VERSION_XML_PARENT_TAG) );
	if( NULL == tag )
		return FALSE;

	TiXmlElement* versionElement = (TiXmlElement*)tag->FirstChild( CT2CA(VERSION_XML_VERSION_TAG) );
	if( NULL == versionElement )
		return FALSE;

	version = versionElement->GetText();

	TiXmlElement* dllUrlElement = (TiXmlElement*)tag->FirstChild( CT2CA(VERSION_XML_DLLURL_TAG) );
	if( NULL == dllUrlElement )
		return FALSE;

	url = dllUrlElement->GetText();
	//delete xmlDoc;

	return TRUE;
}

AgentDLLWrapper* AgentDLLManager::Find( const TCHAR *guid )
{
	CString path = GetGuidInterfaceFolderName(guid);
	CString version, dllname;

	if( FALSE == GetDLLVersionAndPath(path, version, dllname) )
	{
		return NULL;
	}
	
	CString dllpath = path + dllname;
	if( _access( CT2CA(dllpath), 00) == -1 )
	{
		return NULL;
	}

	AgentDLLWrapper *agentDLLWrapper = new AgentDLLWrapper(dllpath);
	if( NULL == agentDLLWrapper )
		return NULL;

	TCHAR agent_dll_guid[128];
	agentDLLWrapper->GetGuid(agent_dll_guid, 128);

	if( _tcsncmp( guid, agent_dll_guid, 128 ) != 0 )
	{
		Unregister(agentDLLWrapper);
		return NULL;
	}
	
	return agentDLLWrapper;
}

static BOOL GetDownloadFileName( const TCHAR *url, TCHAR *filename, int size)
{
	TCHAR *urldata = (TCHAR *)url;
	int len = lstrlen(urldata);
	while( len > 0 )
	{
		if( '/' == urldata[len-1] )
			break;

		len--;
	}

	if( 0 == len )
		return FALSE;

	ATL::Checked::tcscpy_s( filename, size, &urldata[len] );
	return TRUE;
}

BOOL MakeGuidInterfaceFolder( const TCHAR *folder )
{
	if( _access( CT2CA(folder), 00 ) == -1 )
	{
		if( ERROR_SUCCESS != SHCreateDirectoryEx( NULL, folder, NULL ) )
		{
			DWORD err = GetLastError();
			return FALSE;
		}
	}

	return TRUE;
}

BOOL MakeVersionXML( const TCHAR *version, const TCHAR *dllname, const TCHAR *folder )
{
	CString versionXMLTemplate = _T("<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n");
	versionXMLTemplate += _T("<root>\r\n");
	versionXMLTemplate += _T("\t<agentdll>\r\n");
	versionXMLTemplate += _T("\t\t<version>");
	versionXMLTemplate += version;
	versionXMLTemplate += _T("</version>\r\n");
	versionXMLTemplate += _T("\t\t<dllname>");
	versionXMLTemplate += dllname;
	versionXMLTemplate += _T("</dllname>\r\n");
	versionXMLTemplate += _T("\t</agentdll>\r\n");
	versionXMLTemplate += _T("</root>\r\n");
	CString filename = folder;
	filename += _T("Version.XML");

	HANDLE hFile = ::CreateFile( filename, GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
								CREATE_ALWAYS, 0, 0 );
	if( INVALID_HANDLE_VALUE == hFile )
		return FALSE;

	DWORD dwRead;

	LPSTR data = XMLUtil::Encode_utf8(versionXMLTemplate);
	BOOL ret = ::WriteFile(hFile, data, strlen(data), &dwRead, NULL );
	::CloseHandle(hFile);
	return ret;
}

BOOL AgentDLLManager::CheckSameAgentDLLVersion( const TCHAR *guid, const TCHAR *dllversion )
{
	CString path = GetGuidInterfaceFolderName(guid);
	CString version, dllname;

	if( FALSE == GetDLLVersionAndPath( path, version, dllname ) )
	{
		return FALSE;
	}

	if( version != dllversion )
		return FALSE;

	AgentDLLWrapper *agentdll = Find( guid );
	BOOL bRet = FALSE;

	if( NULL != agentdll)
	{
		bRet = TRUE;
		Unregister(agentdll);
	}
	return bRet;
}

BOOL AgentDLLManager::CreateAgentDLL( const TCHAR *guid, const TCHAR *dllversion, const TCHAR *dllurl)
{
	CString guidInterfaceFolder = GetGuidInterfaceFolderName(guid);
	if ( FALSE == MakeGuidInterfaceFolder(guidInterfaceFolder) )
		return FALSE;

	CString targetname = guidInterfaceFolder;
	TCHAR filename[1024];
	GetDownloadFileName( dllurl, filename, 1024 );
	targetname += filename;

	BOOL ret;
	tipbase::GenericHTTPClient httpClient;
	ret = httpClient.Request( NULL, dllurl, tipbase::GenericHTTPClient::RequestGetMethod,
		targetname, L"TipInstallerController", NULL);

	if( FALSE == ret )
		return FALSE;

	MakeVersionXML( dllversion, filename, guidInterfaceFolder );
	return TRUE;
}

AgentDLLWrapper *AgentDLLManager::Register( const TCHAR *guid, const TCHAR *url )
{
	// agentdll  xml  ´.
	tipbase::GenericHTTPClient httpClient;
	BOOL ret = httpClient.Request( NULL, url, tipbase::GenericHTTPClient::RequestGetMethod,
		NULL, L"TipInstallerController", NULL);

	if( FALSE == ret )
		return FALSE;

	BYTE *data  = httpClient.QueryHTTPResponse();
	CString dllversion, dllurl;

	if( FALSE == GetDLLVersionAndUrl(data, dllversion, dllurl) )
		return FALSE;

	if( FALSE == CheckSameAgentDLLVersion(guid, dllversion) )
	{
		if( FALSE == CreateAgentDLL(guid, dllversion, dllurl) )
			return FALSE;
	}

	return Find( guid );

}

BOOL AgentDLLManager::Unregister( AgentDLLWrapper *agentdll)
{
	if ( NULL == agentdll )
		return FALSE;

	delete agentdll;
	return TRUE;
}